from datasets import concatenate_datasets, load_from_disk, Dataset
from pathlib import Path
import polars as pl
from re import search
import duckdb


path = Path('/media/kevin/75b198db-809a-4bd2-a97c-e52daa6b3a2d/item_match')
path_tmp = {
    'db': {'db_array': path / 'db_array', 'db_ds': path / 'db_ds'},
    'q': {'q_array': path / 'q_array', 'q_ds': path / 'q_ds'},
}

dataset_db = concatenate_datasets([
    load_from_disk(str(f)) for f in sorted(path_tmp['db']['db_ds'].glob('*'))
])
path_query = Path('/media/kevin/75b198db-809a-4bd2-a97c-e52daa6b3a2d/item_match/q_clean.parquet')
query = f"""
select * 
from read_parquet('{path_query}') 
"""
df_q = duckdb.sql(query).pl()


# Add index
# col_embed = 'dense_embed'
# path_index = path / 'index'
# dataset_db.load_faiss_index(col_embed, path_index / f'ip.index')
#
# # Dataset query shard
# dataset_q = concatenate_datasets([
#     load_from_disk(str(f)) for f in sorted(path_tmp['q']['q_ds'].glob('*'))
# ])

# Batch query
path_result = path / 'result'
# make_dir(self.path_result)

# query_batch_size = 50_000
# top_k = 100
# num_batches = len(dataset_q) // query_batch_size
# print(num_batches, query_batch_size)
# # logger.info(f'[Matching] Start retrieve: num batches {num_batches}')
# # start = perf_counter()
# for idx, i in enumerate(range(0, len(dataset_q), query_batch_size)):
#     # init
#     file_name_result = path_result / f'result_{idx}.parquet'
#     file_name_score = path_result / f'score_{idx}.parquet'
#     if file_name_result.exists():
#         continue
# 
#     # start
#     # start_batch = perf_counter()
#     if i + query_batch_size >= len(dataset_q):
#         batched_queries = dataset_q[i:]
#     else:
#         batched_queries = dataset_q[i:i + query_batch_size]
# 
#     # query
#     score, result = dataset_db.get_nearest_examples_batch(
#         col_embed,
#         batched_queries[col_embed],
#         k=top_k
#     )
#     # export
#     dict_ = {f'score_{col_embed}': [list(i) for i in score]}
#     df_score = pl.DataFrame(dict_)
#     df_score.write_parquet(file_name_score)
#     df_result = pl.DataFrame(result).drop([col_embed])
#     df_result.write_parquet(file_name_result)
# 
#     # log
#     # print(f"[Matching] Batch {idx}/{num_batches} match result shape: {df_result.shape} "
#     #       f"{perf_counter() - start_batch:,.2f}s")
#     del score, result, df_score, df_result

df_score = (
    pl.concat([pl.read_parquet(f) for f in sorted(path_result.glob('score*.parquet'))])
)
df_result = (
    pl.concat([pl.read_parquet(f) for f in sorted(path_result.glob('result*.parquet'))])
)
df_match = pl.concat([df_q, df_result, df_score], how='horizontal')
col_explode = [i for i in df_match.columns if search('db|score', i)]
df_match = df_match.explode(col_explode)
path_match_result = path / 'result_match'
export_type = 'csv'
cat = 'all'
file_name = path_match_result / f'{cat}.{export_type}'
df_match.write_csv(file_name)



from pathlib import Path
import polars as pl


def rm_all_folder(path: Path) -> None:
    for child in path.glob('*'):
        if child.is_file():
            child.unlink()
        else:
            rm_all_folder(child)
    path.rmdir()


def make_dir(folder_name: str | Path) -> None:
    """Make a directory if it doesn't exist"""
    if isinstance(folder_name, str):
        folder_name = Path(folder_name)
    if not folder_name.exists():
        folder_name.mkdir(parents=True, exist_ok=True)


def clean_check(data: pl.DataFrame, pct: bool = False, verbose: bool = False) -> dict:
    null_check_dict = data.null_count().to_dicts()[0].items()
    null_check = {i: v for i, v in null_check_dict if v > 0}
    if pct:
        null_check = {i: v / data.shape[0] for i, v in null_check_dict if v > 0}

    if verbose:
        print(f"- Null columns: {null_check}")
    dict_ = {
        'null': null_check,
    }
    return dict_



import polars as pl
import duckdb
import sys
import re
from loguru import logger

logger.remove()
logger.add(sys.stdout, colorize=True, format='<level>{level}</level> | <cyan>{function}</cyan> | <level>{message}</level>')


class PipelineText:
    def __init__(self, mode: str = ''):
        self.mode = mode

    @staticmethod
    def clean_text(data: pl.DataFrame, col: str = 'item_name') -> pl.DataFrame:
        regex = "[\(\[\<\"].*?[\)\]\>\"]"
        return data.with_columns(
            pl.col(col).map_elements(
                lambda x: re.sub(regex, "", x).lower().rstrip('.').strip(), return_dtype=pl.String
            )
            .alias(f'{col.lower()}_clean')
        )

    def run(self, data, key_col: list = None):
        # load data
        query = f"""select * from data"""
        df = duckdb.sql(query).pl()
        logger.info(f'[Data] Base Data {self.mode}: {df.shape}')

        df = (
            df
            .pipe(PipelineText.clean_text)
            .drop_nulls(subset=key_col)
            .select(pl.all().name.prefix(f'{self.mode}_'))
        )
        logger.info(f'[Data] Join Data {self.mode}: {df.shape}')
        return df
